@@ -24,8 +24,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
 
 #include "core.h"
 #include "print.h"
@@ -75,7 +75,7 @@ mdc800_sendInitialCommand (Camera *camera, char* answer)
 	    }
 	    printCoreError("Probing RS232 Baudrate with %d fails.\n",baud_rates[rate]);
 	}
-	printCoreError ("Probing failed completly.\n");
+	printCoreError ("Probing failed completely.\n");
 	return GP_ERROR_IO;
 }
 
