@@ -1,19 +1,22 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
-/* gphoto2-port.c
+/** \file
  * 
- * Copyright  2001 Lutz Mller <lutz@users.sf.net>
- * Copyright  1999 Scott Fritzinger <scottf@unr.edu>
+ * \author Copyright 2001 Lutz Mller <lutz@users.sf.net>
+ * \author Copyright 1999 Scott Fritzinger <scottf@unr.edu>
  *
+ * \par License
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
+ * \par
  * This library is distributed in the hope that it will be useful, 
  * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details. 
  *
+ * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -21,7 +24,6 @@
  */
 
 #include "config.h"
-#include "gphoto2-port.h"
 
 #include <stdarg.h>
 #include <stdlib.h>
@@ -30,10 +32,9 @@
 
 #include <ltdl.h>
 
-#include "gphoto2-port-result.h"
-#include "gphoto2-port-info-list.h"
-#include "gphoto2-port-library.h"
-#include "gphoto2-port-log.h"
+#include <gphoto2/gphoto2-port-result.h>
+#include <gphoto2/gphoto2-port-library.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -47,6 +48,7 @@
 #else
 #  define _(String) (String)
 #  define N_(String) (String)
+#  define ngettext(String1,String2,Count) ((Count==1)?String1:String2)
 #endif
 
 #define CHECK_RESULT(result) {int r=(result); if (r<0) return (r);}
@@ -55,27 +57,28 @@
 #define CHECK_INIT(p) {if (!(p)->pc->ops) {gp_port_set_error ((p), _("The port has not yet been initialized")); return (GP_ERROR_BAD_PARAMETERS);}}
 
 /**
- * GPPortPrivateCore:
- *
+ * \brief Internal private libgphoto2_port data.
  * This structure contains private data.
  **/
 struct _GPPortPrivateCore {
-	char error[2048];
+	char error[2048];	/**< Internal kept error message. */
 
-	GPPortInfo info;
-	GPPortOperations *ops;
-	lt_dlhandle lh;
+	GPPortInfo info;	/**< Internal port information of this port. */
+	GPPortOperations *ops;	/**< Internal port operations. */
+	lt_dlhandle lh;		/**< Internal libtool library handle. */
 };
 
 /**
- * gp_port_new:
- * @port:
+ * \brief Create new GPPort
  *
- * Allocates the memory for a new #GPPort. After you called this function, 
+ * Allocate and initialize the memory for a new #GPPort.
+ *
+ * After you called this function, 
  * you probably want to call #gp_port_set_info in order to make the newly
- * created @port functional.
+ * created port functional.
  *
- * Return value: a gphoto2 error code
+ * \param port Pointer the #GPPort* pointer
+ * \return a gphoto2 error code
  **/
 int
 gp_port_new (GPPort **port)
@@ -124,14 +127,17 @@ gp_port_exit (GPPort *port)
 }
 
 /**
- * gp_port_set_info:
- * @port: a #GPPort
- * @info: the #GPPortInfo
+ * \brief Configure a port
  *
- * Makes a @port functional. After calling this function, you can 
- * access the port using for example #gp_port_open.
+ * Makes a port functional by passing in the necessary path
+ * information (from the serial:/dev/ttyS0 or similar variables).
+ * After calling this function, you can access the port using for
+ * example gp_port_open().
+ * 
+ * \param port a GPPort
+ * \param info the GPPortInfo to set
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_set_info (GPPort *port, GPPortInfo info)
@@ -204,6 +210,16 @@ gp_port_set_info (GPPort *port, GPPortInfo info)
 			strchr(info.path, ':') + 1, 
 			sizeof(port->settings.disk.mountpoint));
 		break;
+	case GP_PORT_USB_DISK_DIRECT:
+		snprintf(port->settings.usbdiskdirect.path,
+			 sizeof(port->settings.usbdiskdirect.path), "%s",
+			 strchr(info.path, ':') + 1);
+		break;
+	case GP_PORT_USB_SCSI:
+		snprintf(port->settings.usbscsi.path,
+			 sizeof(port->settings.usbscsi.path), "%s",
+			 strchr(info.path, ':') + 1);
+		break;
 	default:
 		/* Nothing in here */
 		break;
@@ -214,13 +230,13 @@ gp_port_set_info (GPPort *port, GPPortInfo info)
 }
 
 /**
- * gp_port_get_info:
- * @port: a #GPPort
- * @info: a #GPPortInfo
+ * \brief Retreives information about the port.
  *
- * Retreives @info about the port.
+ * Retrieves the informations set by gp_port_set_info().
  *
- * Return value: a gphoto2 error code
+ * \param port a GPPort
+ * \param info GPPortInfo
+ * \return a gphoto2 error code
  **/
 int
 gp_port_get_info (GPPort *port, GPPortInfo *info)
@@ -233,13 +249,13 @@ gp_port_get_info (GPPort *port, GPPortInfo *info)
 }
 
 /**
- * gp_port_open:
- * @port: a #GPPort
+ * \brief Open a port.
+ * \param port a #GPPort
  *
  * Opens a port which should have been created with #gp_port_new and 
  * configured with #gp_port_set_info and #gp_port_set_settings
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_open (GPPort *port)
@@ -258,13 +274,13 @@ gp_port_open (GPPort *port)
 }
 
 /**
- * gp_port_close:
- * @port: a #GPPort
+ * \brief Close a port.
+ * \param port a #GPPort
  *
- * Closes a @port temporarily. It can afterwards be reopened using
+ * Closes a port temporarily. It can afterwards be reopened using
  * #gp_port_open.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_close (GPPort *port)
@@ -281,12 +297,12 @@ gp_port_close (GPPort *port)
 }
 
 /**
- * gp_port_free:
- * @port: a #GPPort
+ * \brief Free the port structure
+ * \param port a #GPPort
  *
- * Closes the @port and frees the memory.
+ * Closes the port and frees the memory.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_free (GPPort *port)
@@ -322,22 +338,26 @@ gp_port_free (GPPort *port)
 }
 
 /**
- * gp_port_write:
- * @port: a #GPPort
- * @data: the data to write to the port
- * @size: the number of bytes to write to the port
+ * \brief Writes a specified amount of data to a port.
+
+ * \param port a #GPPort
+ * \param data the data to write to the port
+ * \param size the number of bytes to write to the port
  *
- * Writes a specified amount of @data to a @port.
+ * Writes data to the port. On non-serial ports the amount of data
+ * written is returned (and not just GP_OK).
  *
- * Return value: a gphoto2 error code
+ * \return a negative gphoto2 error code or the amount of data written.
  **/
 int
 gp_port_write (GPPort *port, const char *data, int size)
 {
 	int retval;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Writing %i=0x%x byte(s) "
-		"to port..."), size, size);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+		"Writing %i=0x%x byte to port...",
+		"Writing %i=0x%x bytes to port...",
+		size), size, size);
 
 	CHECK_NULL (port && data);
 	CHECK_INIT (port);
@@ -349,28 +369,35 @@ gp_port_write (GPPort *port, const char *data, int size)
 	retval = port->pc->ops->write (port, data, size);
 	CHECK_RESULT (retval);
 	if ((port->type != GP_PORT_SERIAL) && (retval != size))
-		gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Could only write %i "
-			"out of %i byte(s)"), retval, size);
-
-	return (retval);
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Could only write %i out of %i byte",
+			"Could only write %i out of %i bytes",
+			size
+		), retval, size);
+	return retval;
 }
 
 /**
- * gp_port_read:
- * @port: a #GPPort
- * @data: a pointer to an allocated buffer
- * @size: the number of bytes that should be read
+ * \brief Read data from port
+ *
+ * \param port a #GPPort
+ * \param data a pointer to an allocated buffer
+ * \param size the number of bytes that should be read
  *
  * Reads a specified number of bytes from the port into the supplied buffer.
+ * It returns the number of bytes read or a negative error code.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code or the amount of data read
  **/
 int
 gp_port_read (GPPort *port, char *data, int size)
 {
         int retval;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Reading %i=0x%x bytes from port..."),
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+		"Reading %i=0x%x byte from port...",
+		"Reading %i=0x%x bytes from port...",
+		size),
 		size, size);
 
 	CHECK_NULL (port);
@@ -381,33 +408,38 @@ gp_port_read (GPPort *port, char *data, int size)
 	retval = port->pc->ops->read (port, data, size);
 	CHECK_RESULT (retval);
 	if (retval != size)
-		gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Could only read %i "
-			"out of %i byte(s)"), retval, size);
-
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Could only read %i out of %i byte",
+			"Could only read %i out of %i bytes",
+			size
+		), retval, size);
 	gp_log_data ("gphoto2-port", data, retval);
-
-	return (retval);
+	return retval;
 }
 
 /**
- * gp_port_check_int:
- * @port: a #GPPort
- * @data: a pointer to an allocated buffer
- * @size: the number of bytes that should be read
+ * \brief Check for intterupt.
  *
- * Reads a specified number of bytes from the inerrupt endpoint
+ * \param port a GPPort
+ * \param data a pointer to an allocated buffer
+ * \param size the number of bytes that should be read
+ *
+ * Reads a specified number of bytes from the interrupt endpoint
  * into the supplied buffer.
  * Function waits port->timeout miliseconds for data on interrupt endpoint.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_check_int (GPPort *port, char *data, int size)
 {
         int retval;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Reading %i=0x%x bytes from interrupt endpoint..."),
-		size, size);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+		"Reading %i=0x%x byte from interrupt endpoint...",
+		"Reading %i=0x%x bytes from interrupt endpoint...",
+		size
+		), size, size);
 
 	CHECK_NULL (port);
 	CHECK_INIT (port);
@@ -417,27 +449,28 @@ gp_port_check_int (GPPort *port, char *data, int size)
 	retval = port->pc->ops->check_int (port, data, size, port->timeout);
 	CHECK_RESULT (retval);
 	if (retval != size)
-		gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Could only read %i "
-			"out of %i byte(s)"), retval, size);
-
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Could only read %i out of %i byte",
+			"Could only read %i out of %i bytes",
+			size), retval, size);
 	gp_log_data ("gphoto2-port", data, retval);
-
-	return (retval);
+	return retval;
 }
 
+/** The timeout in milliseconds for fast interrupt reads. */
+#define FAST_TIMEOUT	50
 /**
- * gp_port_check_int_fast:
- * @port: a #GPPort
- * @data: a pointer to an allocated buffer
- * @size: the number of bytes that should be read
+ * \brief Check for interrupt without wait
+ * \param port a GPPort
+ * \param data a pointer to an allocated buffer
+ * \param size the number of bytes that should be read
  *
  * Reads a specified number of bytes from the inerrupt endpoint
  * into the supplied buffer.
  * Function waits 50 miliseconds for data on interrupt endpoint.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
-#define FAST_TIMEOUT	50
 int
 gp_port_check_int_fast (GPPort *port, char *data, int size)
 {
@@ -456,8 +489,11 @@ gp_port_check_int_fast (GPPort *port, char *data, int size)
 #else
 	if (retval != size )
 #endif
-		gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Could only read %i "
-			"out of %i byte(s)"), retval, size);
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Could only read %i out of %i byte",
+			"Could only read %i out of %i bytes",
+			size
+		), retval, size);
 
 #ifdef IGNORE_EMPTY_INTR_READS
 	if ( retval != 0 ) {
@@ -465,34 +501,39 @@ gp_port_check_int_fast (GPPort *port, char *data, int size)
 		/* For Canon cameras, we will make lots of
 		   reads that will return zero length. Don't
 		   bother to log them as errors. */
-		gp_log (GP_LOG_DEBUG, "gphoto2-port",
-			_("Reading %i=0x%x bytes from interrupt endpoint (fast)..."),
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Reading %i=0x%x byte from interrupt endpoint (fast)...",
+			"Reading %i=0x%x bytes from interrupt endpoint (fast)...",
+			size
+			),
 			size, size);
 		gp_log_data ("gphoto2-port", data, retval);
 #ifdef IGNORE_EMPTY_INTR_READS
 	}
 #endif
-
-	return (retval);
+	return retval;
 }
 
 
 /**
- * gp_port_set_timeout:
- * @port: a #GPPort
- * @timeout: the timeout
+ * \brief Set timeout of port 
+ * \param port a #GPPort
+ * \param timeout the timeout
  *
- * Sets the @timeout of a @port. #gp_port_read will wait @timeout milliseconds
+ * Sets the timeout of a port. #gp_port_read will wait timeout milliseconds
  * for data. If no data will be received in that period, %GP_ERROR_TIMEOUT
  * will be returned.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_set_timeout (GPPort *port, int timeout)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Setting timeout to %i "
-		"millisecond(s)..."), timeout);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+		"Setting timeout to %i millisecond...",
+		"Setting timeout to %i milliseconds...",
+		timeout
+	), timeout);
 
 	CHECK_NULL (port);
 
@@ -501,14 +542,14 @@ gp_port_set_timeout (GPPort *port, int timeout)
         return GP_OK;
 }
 
-/* Deprecated */
+/** Deprecated */
 int gp_port_timeout_set (GPPort *, int);
 int gp_port_timeout_set (GPPort *port, int timeout)
 {
 	return (gp_port_set_timeout (port, timeout));
 }
 
-/* Deprecated */
+/** Deprecated */
 int gp_port_timeout_get (GPPort *, int *);
 int gp_port_timeout_get (GPPort *port, int *timeout)
 {
@@ -516,13 +557,13 @@ int gp_port_timeout_get (GPPort *port, int *timeout)
 }
 
 /**
- * gp_port_get_timeout:
- * @port: a #GPPort
- * @timeout:
+ * \brief Get the current port timeout.
+ * \param port a #GPPort
+ * \param timeout pointer to timeout
  *
  * Retreives the current timeout of the port.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_get_timeout (GPPort *port, int *timeout)
@@ -540,15 +581,15 @@ gp_port_get_timeout (GPPort *port, int *timeout)
 }
 
 /**
- * gp_port_set_settings:
- * @port: a #GPPort
- * @settings: the #GPPortSettings to be set
+ * \brief Set port settings
+ * \param port a #GPPort
+ * \param settings the #GPPortSettings to be set
  *
  * Adjusts the settings of a port. You should always call
  * #gp_port_get_settings, adjust the values depending on the type of the port,
  * and then call #gp_port_set_settings.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_set_settings (GPPort *port, GPPortSettings settings)
@@ -570,12 +611,13 @@ gp_port_set_settings (GPPort *port, GPPortSettings settings)
         return (GP_OK);
 }
 
-/* Deprecated */
+/** Deprecated */
 int gp_port_settings_get (GPPort *, GPPortSettings *);
 int gp_port_settings_get (GPPort *port, GPPortSettings *settings)
 {
 	return (gp_port_get_settings (port, settings));
 }
+/** Deprecated */
 int gp_port_settings_set (GPPort *, GPPortSettings);
 int gp_port_settings_set (GPPort *port, GPPortSettings settings)
 {
@@ -583,13 +625,13 @@ int gp_port_settings_set (GPPort *port, GPPortSettings settings)
 }
 
 /**
- * gp_port_get_settings:
- * @port: a #GPPort
- * @settings:
+ * \brief Get the current port settings.
+ * \param port a #GPPort
+ * \param settings pointer to the retrieved settings
  *
- * Retreives the current @settings of a @port.
+ * Retreives the current settings of a port.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_get_settings (GPPort *port, GPPortSettings *settings)
@@ -601,6 +643,15 @@ gp_port_get_settings (GPPort *port, GPPortSettings *settings)
         return GP_OK;
 }
 
+/**
+ * \brief Get setting of specific serial PIN
+ *
+ * \param port a GPPort
+ * \param pin the serial pin to be retrieved
+ * \param level the setting of the pin
+ * 
+ * \return a gphoto2 error code
+ */
 int
 gp_port_get_pin (GPPort *port, GPPin pin, GPLevel *level)
 {
@@ -644,6 +695,17 @@ static struct {
 	{0, NULL}
 };
 
+/**
+ * \brief Set specified serial PIN to value
+ *
+ * \param port a GPPort
+ * \param pin the serial pin to be retrieved
+ * \param level the setting of the pin
+ *
+ * Pulls the specified pin of a serial port to the specified level.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_set_pin (GPPort *port, GPPin pin, GPLevel level)
 {
@@ -669,11 +731,24 @@ gp_port_set_pin (GPPort *port, GPPin pin, GPLevel level)
 	return (GP_OK);
 }
 
+/**
+ * \brief Send a break over a serial port
+ *
+ * \param port a GPPort
+ * \param duration duration of break in milliseconds
+ *
+ * Sends a break with the specified duration in milliseconds.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_send_break (GPPort *port, int duration)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Sending break (%i "
-		"milliseconds)..."), duration);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+		"Sending break (%i millisecond)...",
+		"Sending break (%i milliseconds)...",
+		duration
+	), duration);
 
 	CHECK_NULL (port);
 	CHECK_INIT (port);
@@ -684,6 +759,17 @@ gp_port_send_break (GPPort *port, int duration)
 	return (GP_OK);
 }
 
+/**
+ * \brief Flush data on serial port
+ *
+ * \param port a GPPort
+ * \param direction the direction of the flush
+ *
+ * Flushes the serial output or input (depending on direction)
+ * of the serial port.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_flush (GPPort *port, int direction)
 {
@@ -701,6 +787,17 @@ gp_port_flush (GPPort *port, int direction)
 /* USB-specific functions */
 /* ------------------------------------------------------------------ */
 
+/**
+ * \brief Find USB device by vendor/product
+ *
+ * \param port a GPPort
+ * \param idvendor USB vendor id
+ * \param idproduct USB product id
+ *
+ * Find the USB device with the specified vendor:product id pair.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_usb_find_device (GPPort *port, int idvendor, int idproduct)
 {
@@ -713,6 +810,18 @@ gp_port_usb_find_device (GPPort *port, int idvendor, int idproduct)
         return (GP_OK);
 }
 
+/**
+ * \brief Find USB device by interface class
+ *
+ * \param port a GPPort
+ * \param mainclass the USB interface class
+ * \param subclass the USB interface subclass
+ * \param protocol the USB interface protocol
+ *
+ * Find the USB device with the specified vendor:product id pair.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_usb_find_device_by_class (GPPort *port, int mainclass, int subclass, int protocol)
 {
@@ -725,6 +834,16 @@ gp_port_usb_find_device_by_class (GPPort *port, int mainclass, int subclass, int
         return (GP_OK);
 }
 
+/**
+ * \brief Clear USB endpoint HALT condition
+ *
+ * \param port a GPPort
+ * \param ep endpoint to clear HALT
+ *
+ * Clears the HALT (stall?) endpoint condition of the specified endpoint.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_usb_clear_halt (GPPort *port, int ep)
 {
@@ -739,6 +858,20 @@ gp_port_usb_clear_halt (GPPort *port, int ep)
         return (GP_OK);
 }
 
+/**
+ * \brief Send a USB control message with output data
+ *
+ * \param port a GPPort
+ * \param request control request code
+ * \param value control value
+ * \param index control index
+ * \param bytes pointer to data
+ * \param size size of the data
+ *
+ * Sends a specific USB control command and write associated data.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_usb_msg_write (GPPort *port, int request, int value, int index,
 	char *bytes, int size)
@@ -760,6 +893,20 @@ gp_port_usb_msg_write (GPPort *port, int request, int value, int index,
         return (retval);
 }
 
+/**
+ * \brief Send a USB control message with input data
+ *
+ * \param port a GPPort
+ * \param request control request code
+ * \param value control value
+ * \param index control index
+ * \param bytes pointer to data
+ * \param size size of the data
+ *
+ * Sends a specific USB interface control command and read associated data.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_usb_msg_read (GPPort *port, int request, int value, int index,
 	char *bytes, int size)
@@ -773,24 +920,38 @@ gp_port_usb_msg_read (GPPort *port, int request, int value, int index,
 	CHECK_NULL (port);
 	CHECK_INIT (port);
 
-	CHECK_SUPP (port, _("msg_read"), port->pc->ops->msg_read);
+	CHECK_SUPP (port, "msg_read", port->pc->ops->msg_read);
         retval = port->pc->ops->msg_read (port, request, value, index, bytes, size);
 	CHECK_RESULT (retval);
 
 	if (retval != size)
-		gp_log (GP_LOG_DEBUG, "gphoto2-port", _("Could only read %i "
-			"out of %i byte(s)"), retval, size);
-
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Could only read %i out of %i byte",
+			"Could only read %i out of %i bytes",
+			size
+		), retval, size);
 	gp_log_data ("gphoto2-port", bytes, retval);
-
-        return (retval);
+        return retval;
 }
 
 /* 
  * The next two functions handle the request types 0x41 for write 
  * and 0xc1 for read.
  */
-
+/**
+ * \brief Send a USB interface control message with output data
+ *
+ * \param port a GPPort
+ * \param request control request code
+ * \param value control value
+ * \param index control index
+ * \param bytes pointer to data
+ * \param size size of the data
+ *
+ * Sends a specific USB interface control command and write associated data.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_usb_msg_interface_write (GPPort *port, int request, 
 	int value, int index, char *bytes, int size)
@@ -814,6 +975,20 @@ gp_port_usb_msg_interface_write (GPPort *port, int request,
 }
 
 
+/**
+ * \brief Send a USB interface control message with input data
+ *
+ * \param port a GPPort
+ * \param request control request code
+ * \param value control value
+ * \param index control index
+ * \param bytes pointer to data
+ * \param size size of the data
+ *
+ * Sends a specific USB control command and read associated data.
+ *
+ * \return a gphoto2 error code
+ */
 int
 gp_port_usb_msg_interface_read (GPPort *port, int request, int value, int index,
 	char *bytes, int size)
@@ -833,24 +1008,188 @@ gp_port_usb_msg_interface_read (GPPort *port, int request, int value, int index,
 	CHECK_RESULT (retval);
 
 	if (retval != size)
-		gp_log (GP_LOG_DEBUG, "gphoto2-port", "Could only read %i "
-			"out of %i byte(s)", retval, size);
-
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Could only read %i out of %i byte",
+			"Could only read %i out of %i bytes",
+			size
+		),retval, size);
 	gp_log_data ("gphoto2-port", bytes, retval);
+        return retval;
+}
+
+
+/* 
+ * The next two functions handle the request types 0x21 for write 
+ * and 0xa1 for read.
+ */
+
+/**
+ * \brief Send a USB class control message with output data
+ *
+ * \param port a GPPort
+ * \param request control request code
+ * \param value control value
+ * \param index control index
+ * \param bytes pointer to data
+ * \param size size of the data
+ *
+ * Sends a specific USB class control command and write associated data.
+ *
+ * \return a gphoto2 error code
+ */
+int
+gp_port_usb_msg_class_write (GPPort *port, int request, 
+	int value, int index, char *bytes, int size)
+{
+        int retval;
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Writing message "
+		"(request=0x%x value=0x%x index=0x%x size=%i=0x%x)...",
+		request, value, index, size, size);
+	gp_log_data ("gphoto2-port", bytes, size);
+
+	CHECK_NULL (port);
+	CHECK_INIT (port);
+
+	CHECK_SUPP (port, "msg_build", port->pc->ops->msg_class_write);
+        retval = port->pc->ops->msg_class_write(port, request, 
+        		value, index, bytes, size);
+	CHECK_RESULT (retval);
 
         return (retval);
 }
 
 
 /**
- * gp_port_set_error:
- * @port: a #GPPort
- * @format:
- * @...:
+ * \brief Send a USB class control message with input data
+ *
+ * \param port a GPPort
+ * \param request control request code
+ * \param value control value
+ * \param index control index
+ * \param bytes pointer to data
+ * \param size size of the data
+ *
+ * Sends a specific USB class control command and read associated data.
+ *
+ * \return a gphoto2 error code
+ */
+int
+gp_port_usb_msg_class_read (GPPort *port, int request, int value, int index,
+	char *bytes, int size)
+{
+        int retval;
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Reading message "
+		"(request=0x%x value=0x%x index=0x%x size=%i=0x%x)...",
+		request, value, index, size, size);
+
+	CHECK_NULL (port);
+	CHECK_INIT (port);
+
+	CHECK_SUPP (port, "msg_read", port->pc->ops->msg_class_read);
+        retval = port->pc->ops->msg_class_read (port, request, 
+        		value, index, bytes, size);
+	CHECK_RESULT (retval);
+
+	if (retval != size)
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", ngettext(
+			"Could only read %i out of %i byte",
+			"Could only read %i out of %i bytes",
+			size
+		), retval, size);
+	gp_log_data ("gphoto2-port", bytes, retval);
+        return retval;
+}
+
+/**
+ * \brief Seek on a port (for usb disk direct ports)
+ *
+ * \param port a #GPPort
+ * \param offset offset to seek to
+ * \param whence the underlying lseek call whence parameter
+ *
+ * Seeks to a specific offset on the usb disk
+ *
+ * \return a gphoto2 error code
+ **/
+int
+gp_port_seek (GPPort *port, int offset, int whence)
+{
+	int retval;
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Seeking to: %d whence: %d",
+		offset, whence);
+
+	CHECK_NULL (port);
+	CHECK_INIT (port);
+
+	CHECK_SUPP (port, "seek", port->pc->ops->seek);
+	retval = port->pc->ops->seek (port, offset, whence);
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Seek result: %d", retval);
+
+	return retval;
+}
+
+/**
+ * \brief Send a SCSI command to a port (for usb scsi ports)
+ *
+ * \param port a #GPPort
+ * \param to_dev data direction, set to 1 for a scsi cmd which sends
+ *        data to the device, set to 0 for cmds which read data from the dev.
+ * \param cmd buffer holding the command to send
+ * \param cmd_size sizeof cmd buffer
+ * \param sense buffer for returning scsi sense information
+ * \param sense_size sizeof sense buffer
+ * \param data buffer containing informatino to write to the device
+ *        (to_dev is 1), or to store data read from the device (to_dev 0).
+ *
+ * Send a SCSI command to a usb scsi port attached device.
+ *
+ * \return a gphoto2 error code
+ **/
+int gp_port_send_scsi_cmd (GPPort *port, int to_dev,
+				char *cmd, int cmd_size,
+				char *sense, int sense_size,
+				char *data, int data_size)
+{
+	int retval;
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Sending scsi cmd:");
+	gp_log_data ("gphoto2-port", cmd, cmd_size);
+	if (to_dev && data_size) {
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", "scsi cmd data:");
+		gp_log_data ("gphoto2-port", data, data_size);
+	}
+
+	CHECK_NULL (port);
+	CHECK_INIT (port);
+
+	CHECK_SUPP (port, "send_scsi_cmd", port->pc->ops->send_scsi_cmd);
+	retval = port->pc->ops->send_scsi_cmd (port, to_dev, cmd, cmd_size,
+					sense, sense_size, data, data_size);
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "scsi cmd result: %d, sense:",
+		retval);
+	gp_log_data ("gphoto2-port", sense, sense_size);
+	if (!to_dev && data_size) {
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", "scsi cmd data:");
+		gp_log_data ("gphoto2-port", data, data_size);
+	}
+
+	return retval;
+}
+
+/**
+ * \brief Set verbose port error message
+ * \param port a #GPPort
+ * \param format printf style format string
+ * \param ... variable arguments depending on format string
  *
  * Sets an error message that can later be retrieved using #gp_port_get_error.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_port_set_error (GPPort *port, const char *format, ...)
@@ -873,15 +1212,15 @@ gp_port_set_error (GPPort *port, const char *format, ...)
 }
 
 /**
- * gp_port_get_error:
- * @port: a #GPPort
+ * \brief Get verbose port error message
+ * \param port a #GPPort
  *
- * Retrieves an error message from a @port. If you want to make sure that
+ * Retrieves an error message from a port. If you want to make sure that
  * you get correct error messages, you need to call #gp_port_set_error with
  * an error message of %NULL each time before calling another port-related
  * function of which you want to check the return value.
  *
- * Return value: an error message
+ * \return a translated error message
  **/
 const char *
 gp_port_get_error (GPPort *port)
